#pragma rtGlobals = 1		// modern global method

// PPT Fura Calibration and Isocoefficient calculation
// part of the PPT, the "Patcher's Power Tools"
//  13.05.97 Francisco Mendez

// 13.05.97 new macro: Fura Calibration
// 19.05.97 added help to fura-calibration panel
// 20.05.97 added calculation of isocoefficient
// 23.10.97 macro stores PPT-globals in folder "PPTGlobals"

// ****************************************************

Menu "Macros"
	Submenu "Fura Calibration"
		"Fura Calibration Graph", PPTFuraCalibration ()
		"Isocoefficients...", PPTIsoCoefficients ()
	End
End

// ****************************************************

Proc PPTFuraCalibration ()
	Silent 1; PauseUpdate

	String oldDataFolder = GetDataFolder (1)
	if (! DataFolderExists ("root:PPTGlobals"))
		NewDataFolder /O root:PPTGlobals
	endif

	SetDataFolder root:PPTGlobals

	Variable/G pptRMin = 0.8		// minimum ratio, measured
	Variable/G pptRMax = 6.1		// maximum ratio, measured
	Variable/G pptRInt = 1.3		// intermediate fluorescence ratio, measured
	Variable/G pptKEff				// effective KD, to be calculated

	Variable/G pptEGTA = 5			// concentration/amount of EGTA
	Variable/G pptCaEGTA = 5		// concentration/amount of Ca-EGTA
	Variable/G pptCa					// free calcium in pptKEff-measurement
	Variable/G pptKD = 176e-9		// real KD [M]

	pptCa := pptKD * pptCaEGTA / pptEGTA
	pptKEff := pptCa * (pptRMax - pptRInt) / (pptRInt - pptRMin)

	Make/D/O/N=128 pptRatioWave,pptCalciumWave
	SetScale d 0,0,"M", pptCalciumWave

	// equidistant values from RMin to RMax
	pptRatioWave := pptRMin + x * (pptRMax - pptRMin) / 128

	// Tsien's formula
	pptCalciumWave := pptKEff * (pptRatioWave - pptRMin) / (pptRMax - pptRatioWave)

	PPTFuraCalibrationGraph ()

	SetDataFolder $oldDataFolder
End Proc

// ****************************************************

Proc PPTFuraCalibrationGraph ()
	PauseUpdate; Silent 1		// building window...

	if (! DataFolderExists ("root:PPTGlobals"))
		PPTFuraCalibration ()
	endif
	String oldDataFolder = GetDataFolder (1)
	SetDataFolder root:PPTGlobals

	Variable	panelX = 40
	variable	panelY = 60
	Variable	panelW = 360
	Variable	panelH = 300

	Display /W=(panelX, panelY, panelX + panelW, panelY + panelH) pptCalciumWave vs pptRatioWave as "PPT Fura-2-Calibration"
	ModifyGraph gfSize=12
	ModifyGraph log(left)=1
	Label left "Calcium /\\U"
	Label bottom "Fluorescence Ratio"
	SetAxis left 1e-08,5e-05

	ControlBar 58
	SetVariable vRMin	pos = {4, 3}, size = {100, 15}, title = "R min"
	SetVariable vRMin	limits = {-INF, INF, 1}, value = pptRMin
	SetVariable vRMin	help = {"Fluorescence ratio under Calcium free conditions."}

	SetVariable vRMax	pos = {4, 21}, size = {100, 15}, title = "R max"
	SetVariable vRMax	limits = {-INF, INF, 1}, value = pptRMax
	SetVariable vRMax	help = {"Fluorescence ratio under Calcium saturated conditions."}

	SetVariable vRKEff	pos={4,39},size={100,15},title="R int"
	SetVariable vRKEff	limits={-INF,INF,1},value= pptRInt
	SetVariable vRKEff	help = {"Fluorescence ratio under Calcium buffered conditions."}

	SetVariable vCa	pos = {125, 39}, size = {120, 15}, title = "[Ca2+]"
	SetVariable vCa	limits = {-INF, INF, 1}, value= pptCa
	SetVariable vCa	help = {"Calcium in buffered experiment, according to KD, [EGTA] and [Ca-EGTA]."}

	SetVariable vEGTA	pos = {125, 3}, size = {120, 15}, title = "[EGTA]"
	SetVariable vEGTA	limits = {-INF, INF, 1}, value = pptEGTA
	SetVariable vEGTA	help = {"Amount or concentration of EGTA in buffered experiment."}

	SetVariable vCaEGTA	pos = {125, 21}, size = {120, 15}, title = "[Ca-EGTA]"
	SetVariable vCaEGTA	limits = {-INF, INF, 1}, value = pptCaEGTA
	SetVariable vCaEGTA	help = {"Amount or concentration of Ca-EGTA in buffered experiment."}

	SetVariable vKD	pos = {255, 21}, size = {100, 15}, title ="KD"
	SetVariable vKD	limits = {-INF, INF, 1}, value = pptKD
	SetVariable vKD	help = {"KD of EGTA, literature value 176 nM."}

	SetVariable vKEff	pos = {254, 39}, size = {100, 15}, format = "%.3g", title = "K Eff"
	SetVariable vKEff	limits = {-INF, INF, 1}, value = pptKEff
	SetVariable vKEff	help = {"Calculated effective KD (KEff)."}

	SetDataFolder $oldDataFolder
End Proc

// ****************************************************

Proc PPTIsoCoefficients (f1Wave, f2Wave)
	String	f1Wave, f2Wave
	Prompt	f1Wave, "Choose F1-Wave (360 nm)", popup, "_none_;" + WaveList ("*", ";", "")
	Prompt	f2Wave, "Choose F2-Wave (380 nm)", popup, "_none_;" + WaveList ("*", ";", "")

	PauseUpdate; Silent 1		// building window...
	String	theCmd, theIsoWave

	if ((strlen (f1Wave) <= 0) %| (strlen (f2Wave) <= 0))
		Abort
	endif
	if ((cmpStr (f1Wave, "_none_") == 0) %| (cmpStr (f1Wave, "_none_") == 0))
		Abort "Please select fluorescence waves for wavelength 1 and 2!"
	endif
	if (numPnts ($f1Wave) != numPnts ($f2Wave))
		Abort "The two waves have different number of points!" 
	endif

	if (! DataFolderExists ("root:PPTGlobals"))
		NewDataFolder /O root:PPTGlobals
	endif

	Variable /G root:PPTGlobals:pptRMin = 0.8
	Variable /G root:PPTGlobals:pptRMax = 6.1
	Variable /G root:PPTGlobals:pptKD = 176e-9

	Variable	panelX = 40
	Variable	panelY = 60
	Variable	panelW = 300
	Variable	panelH = 400

	Display /W=(panelX, panelY, panelX + panelW, panelY + panelH) as "PPT Isocoefficient"

	ControlBar 60

	// RMin
	SetVariable vRMin	pos = {40, 4}, size = {110, 15}, title = "R min"
	SetVariable vRMin	limits = {0, INF, 0.02}, value = root:PPTGlobals:pptRMin
	SetVariable vRMin	help = {"Fluorescence ratio under Calcium free conditions."}

	// RMax
	SetVariable vRMax	pos = {40, 22}, size = {110, 15}, title = "R max"
	SetVariable vRMax	limits = {0, INF, 0.2}, value = root:PPTGlobals:pptRMax
	SetVariable vRMax	help = {"Fluorescence ratio under Calcium saturated conditions."}

	// KD
	SetVariable vKD	pos = {40, 40}, size = {110, 15}, title ="KD"
	SetVariable vKD	limits = {0, INF, 10e-9}, value = root:PPTGlobals:pptKD
	SetVariable vKD	help = {"KD of EGTA, literature value 176 nM."}
	
	// Iso-Coefficient
	SetVariable vIsoCoeff	pos = {160, 22}, size = {110, 15}, title = "alpha"
	SetVariable vIsoCoeff	limits = {-INF, INF, 0.01}
	SetVariable vIsoCoeff	help = {"Iso-Coefficient alpha, leading to FIso = F1 + a * F2."}
	String alphaName = f1Wave + "_isoCoeff"
	sprintf theCmd, "Variable/G %s", alphaName
	Execute theCmd
	sprintf theCmd, "SetVariable vIsoCoeff value = %s", alphaName
	Execute theCmd

	// KEff calculated by Isocoefficient Determination
	SetVariable vKEff	pos = {160, 40}, size = {110, 15}, title = "K Eff"
	SetVariable vKEff	format = "%.4g"
	SetVariable vKEff	limits = {0, INF, 1e-6}
	SetVariable vKEff	help = {"KEff determined by KEff = Kd * (a + RMax) / (a + RMin)."}
	String keffName = f1Wave + "_kEff"
	sprintf theCmd, "Variable/G %s", keffName
	Execute theCmd
	sprintf theCmd, "SetVariable vKEff value = %s", keffName
	Execute theCmd

	// Create Iso-Wave and Dependencies
	theIsoWave = f1Wave + "_iso"
	Duplicate/O $f1Wave $theIsoWave
	sprintf theCmd, "%s := %s + %s * %s", theIsoWave, f1Wave, alphaName, f2Wave
	execute theCmd
	sprintf theCmd, "%s := root:PPTGlobals:pptKD * (%s + root:PPTGlobals:pptRMax) / (%s + root:PPTGlobals:pptRMin)", keffName, alphaName, alphaName
	Execute theCmd

	// Create graphs
	ModifyGraph margin(left)=72,gfSize=12
	AppendToGraph/L=l1 $f1Wave
	AppendToGraph/L=l2 $f2Wave
	AppendToGraph/L=l3 $theIsoWave
	ModifyGraph axisEnab(l1)={0.7,1},freePos(l1)={0,bottom},lblPos(l1)=64
	ModifyGraph axisEnab(l2)={0.35,0.65},freePos(l2)={0,bottom},lblPos(l2)=64
	ModifyGraph axisEnab(l3)={0,0.3},freePos(l3)={0,bottom},lblPos(l3)=64
	Label l1 "F\\B1"
	Label l2 "F\\B2"
	Label l3 "F\\Biso"

	ResumeUpdate
End Proc